// This is CSORT.CPP
// Project IMARK - a support program for indexing TeX and LaTeX documents
// (C) Jan Kybic, 1993

// Main module is IMARK.CPP. It contains also compile notes

// This module contains routines for czech sorting
//  see IMARK.H to get functional  description of this module

#include <string.h>
#include <iostream.h>  // for cerr
#include <stdlib.h>    // for exit
#include "CSORT.H"     // header file

// Here we actually declare global variables

unsigned char use_czech=1 ;        // default is to use czech
char (*isletter)(unsigned char c) ;
int  (*compare)(unsigned char *r,unsigned char *s) ;

// Let's define code representation for double letters 'ch' and 'CH'
//  that are to be treated as a single letter

#define CH 256
#define ch 257

// cstab is a source form of a sorting table
//   groups delimited by spaces have primary sorting order
// If two strings are found equal considering only the groups to which
// characters belong, than we take into account also the orders in which 
//  they appear is groups. For example: "a"=="" for the first round, but
//    "">"a" for the second one
// When changing coding scheme, this table must be rewritten

unsigned char cstab[]=
           "a b c  d  e f g h ch i j k l m n  o p q r  s  t "
	   "u v w x y z  A B C  D  E F G H CH I J K L M N  O"
	   "P Q R  S  T  U V W X Y Z " ;

// cstab is converted by 'initcs' to a more suitable form - to array 'csztab'
// Here each czech letter has the group number (s) and its order in a group (p)
// Other characters have s=0 and are compared according to ASCII codes

struct { unsigned char s,p ; } csztab[258] ;

// Get 'cschar' gets a single char from the char string refered to 
//  by parameter 's'. Pointer '*s' is moved onto the next char
// The routin just checks if the coming char isn't 'ch' or 'CH'.
// If yes, it returns their codes #defined previously and advances by 2 chars

int getcschar(unsigned char **s)
{ if (**s=='c')
   if (*(*s+1)=='h') { *s+=2 ; return ch ; }
  if (**s=='C')
    if (*(*s+1)=='H' || *(*s+1)=='h') { *s+=2 ; return CH ; }
  return *((*s)++) ;
}

char isenletter(unsigned char c)
  // test if 'c' is a letter of english alphabet
{ return (c>='A' && c<='Z') || (c>='a' && c<='z') ; }

char iscsletter(unsigned char c)
  // test if 'c' is a letter of czech alphabet  
{ return csztab[c].s ; }

  
int encompare(unsigned char *r,unsigned char *s)
  // compare two char strings using simple ASCII comparation
{
  while (*r && *r==*s) { r++ ; s++ ; } ;
  return *r-*s ;
}

int cscompare(unsigned char *r,unsigned char *s)
  // compare two char strings according to czech 
{ int rc,sc,por=0,pom=0 ;
  while(*r || *s)
   { rc=getcschar(&r) ; sc=getcschar(&s) ;
     if (!csztab[rc].s || !csztab[sc].s)
         if ((por=rc-sc)!=0) return por ;
                        else continue ;
     if ((por=csztab[rc].s-csztab[sc].s)!=0)
        return por ;
     if (!pom) pom=csztab[rc].p-csztab[sc].p ;
   }
  if (pom) return pom ;
}


void initcs(void)            // Generate czech sorting table
{ unsigned char sk=1,por=0,*s=cstab ;
  int i ;
  if (use_czech) {
    for(i=0;i<258;i++) csztab[i].s=csztab[i].p=0 ; // initialization
    while(*s)
     { if (*s==' ') { sk++ ; por=0 ; s++; continue ;}
       csztab[i=getcschar(&s)].s=sk ; csztab[i].p=por ;
       por++ ;
     }}
  isletter=use_czech ? iscsletter : isenletter ;
  compare=use_czech ?  cscompare : encompare ;
}


CString& CString::operator =(const char *s)
{ int l ;
  if ((char *)theString != s)
   { if (len!=(l=strlen((char *)s)+1))
       { if (len) delete theString ;
         len = l ;
         theString = new char [len];
         if (!theString)
           { cerr << "!!! ERROR: Not enough memory.\n" ; exit(1) ; }
       }
     strcpy(theString,(char *)s) ;
   }
   return *this ;
}