// This is IMARK.CPP
// Project IMARK - a support program for indexing TeX and LaTeX documents
// (C) Jan Kybic, 1993

// Compile notes:
// The program was compiled by BC++ 2.0 in compact model using signed chars
// It uses the CLASSLIB object library
// It is necessary to make the following changes in CLASSLIB :
// in STRNG.H: to change attribute in class String from 'private' to 'protected'
// so that it is possible for the derived classes to acces previously
// private items.
// in SORTEDARR.H: to change attribute in class SortedArray from 'private' to
// 'protected' for similar reason

// The program IMARK.EXE is composed from BSORTARR.OBJ, IMCLASS.OBJ, CSORT.OBJ
//  IMLOW.OBJ and TCLASSC.LIB (CLASSLIB library for compact model)

// This is the main module. 
// Here we do no more than just process the command line

#pragma hdrfile d:\data\source\imark.sym // Header file name
#include <stdlib.h>            
#include <ctype.h>
#include "csort.h"
#include "imlow.h"

char *version="IMARK v1.0 , (C) Jan Kybic 1993\n" ;

char help_text[]=
 "This program facitilates indexing of TeX and LaTeX documents.\n"
 "Usage:  IMARK [switches] <command> <LWfile> [TeXfile(s)]\n"
 " ( LWfile contains the list of word to be marked in TeX source files )\n"
 "Switches (preceded by '-' or '/'):\n"
 "    e      use English sorting (default is Czech)\n"
 "    c<cmd> set command to mark chosen words to <cmd> (default is 'index')\n"
 "    o<ext> set extension of output files to <ext> (default is 'NEW')\n"
 "Commands:\n"
 "    h      display this help screen\n"
 "    b      build new LWfile using words from TeX source file(s)\n"
 "    a      add words from TeX source file(s) to existing LWfile\n" 
 "    m      mark words contained in LWfile in TeX source file(s)\n"
 "Commands <A>dd and <B>uild produce alphabetically sorted list of words\n"
 " from given TeX source file(s).\n"
 "You can edit LWfile and delete words you do not want to include in index.\n"
 "Then you execute IMARK with command <M>ark and it will mark all the occurences\n"
 " of words from LWfile in TeX file(s). (like ...word\\index{word})\n"
 "If you put two words on a single line into the LWfile, like ('words word')\n"
 "the first one will be sought for and the second will appear in index.\n"
 "  (like (...words\\index{word}...)\n"
 ;

// errsee displays an error message, advises user to use IMARK H and exits
void errsee(char *message)
{ cerr << "!!! ERROR: " << message <<
   "\n Use IMARK h to get help.\n" ;
  exit(1) ;
}

// And finally function main

int main(int argc,char *argv[])
{ int count ;        // the index of current command line parameter
  char comm=' ' ;    // command

  CString lwname(""),fname("") ; // strings contain file names

  cerr << version ;              // display version
  // Set defaults
  use_czech=1 ;
  output_extension=new CString("new") ; index_command=new CString("index") ;

  // check if we are not out of memory
  // checkObject is a macro from IMLOW.H
  checkObject(output_extension) ;  checkObject(index_command) ;

  // First process the switches, if any
  for(count=1;count<argc;count++)
   if (*argv[count]=='-' || *argv[count]=='/')     // switch char ?
     switch (tolower(*(argv[count]+1)))    	   // yes
      { case 'e' : use_czech=0 ; break ;           // english sorting
        case 'o' : (*output_extension)=argv[count]+2 ; break ;
                           // extension for output file(s)
        case 'c' : (*index_command)=argv[count]+2 ; break ;
                           // index command
        default:   errsee("Invalid switch.") ;
      }
     else { comm=tolower(*(argv[count])) ; break ; }
       // if it has not been switch it must have been command

  if (comm==' ') errsee("Missing command.") ;   // No command ?

  initcs() ;                 // init czech sorting tables if necessary
  LW=new ListOfWords ;       // create new list of words
  checkObject(LW) ;          // check allocations and assignments
  checkObject(output_extension) ;
  checkObject(index_command) ;

  // Did user ask for help ?
  if (comm=='h') { cerr << help_text ; return 0 ; }

  if (comm!='a' && comm!='b' && comm!='m')
                errsee("Invalid command.") ; // Unknown command

  // LWfile name should follow
  if (++count>=argc) errsee("LW file name missing.") ;
  lwname=argv[count++] ;
  // Now there should be at least one TeX source file name
  if (count>=argc) errsee("TeX source file names missing.") ;

  switch(comm)
   { case 'a' : LW->readfrom(lwname) ;      // read existing LWfile
     case 'b' : for(;count<argc;count++)    // for all TeX source files
                 { fname=argv[count] ;
                   LW->makelist(fname) ; }  // add new words to the list
                LW->writeto(lwname) ;       // write the list back to file
                break ;
     case 'm' : LW->readfrom(lwname) ;      // read existing LWfile
                for(;count<argc;count++)    // for all TeX source files
                 { fname=argv[count] ;
                   LW->mark(fname) ; }      // mark words from the list
   }

  cerr << "IMARK finished.\n" ;
  return 0;
}

// End of IMARK.CPP
